@@ -1,9 +1,4 @@
-#define __SP_ATTRIBUTES_C__
-
-/** \file
- * Lookup dictionary for attributes/properties.
- */
-/*
+/**
  * Author:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *
@@ -18,29 +13,45 @@
 #endif
 
 #include <glib.h> // g_assert()
-#include <glib/ghash.h>
 #include "attributes.h"
+#include <iostream>
 
 typedef struct {
     gint code;
     gchar const *name;
 } SPStyleProp;
 
+/**
+ * Lookup dictionary for attributes/properties.
+ */
+
 static SPStyleProp const props[] = {
     {SP_ATTR_INVALID, NULL},
     /* SPObject */
     {SP_ATTR_ID, "id"},
+    {SP_ATTR_STYLE, "style"},
     {SP_ATTR_INKSCAPE_COLLECT, "inkscape:collect"},
     {SP_ATTR_INKSCAPE_LABEL, "inkscape:label"},
+    /* SPRoot */
+    {SP_ATTR_VERSION, "version"},
+    {SP_ATTR_INKSCAPE_VERSION, "inkscape:version"},
+    {SP_ATTR_WIDTH, "width"},
+    {SP_ATTR_HEIGHT, "height"},
+    {SP_ATTR_VIEWBOX, "viewBox"},
+    {SP_ATTR_PRESERVEASPECTRATIO, "preserveAspectRatio"},
+    {SP_ATTR_ONLOAD, "onload"},
+    {SP_ATTR_SODIPODI_DOCNAME, "sodipodi:docname"},
     /* SPItem */
     {SP_ATTR_TRANSFORM, "transform"},
+    {SP_ATTR_SODIPODI_TYPE, "sodipodi:type"},
     {SP_ATTR_SODIPODI_INSENSITIVE, "sodipodi:insensitive"},
-    {SP_ATTR_SODIPODI_NONPRINTABLE, "sodipodi:nonprintable"},
     {SP_ATTR_CONNECTOR_AVOID, "inkscape:connector-avoid"},
-    {SP_ATTR_STYLE, "style"},
+    {SP_ATTR_CONNECTION_POINTS, "inkscape:connection-points"},
     {SP_ATTR_TRANSFORM_CENTER_X, "inkscape:transform-center-x"},
     {SP_ATTR_TRANSFORM_CENTER_Y, "inkscape:transform-center-y"},
     {SP_ATTR_INKSCAPE_PATH_EFFECT, "inkscape:path-effect"},
+    {SP_ATTR_INKSCAPE_HIGHLIGHT_COLOR, "inkscape:highlight-color"},
+    {SP_ATTR_INKSCAPE_SPRAY_ORIGIN, "inkscape:spray-origin"},
     /* SPAnchor */
     {SP_ATTR_XLINK_HREF, "xlink:href"},
     {SP_ATTR_XLINK_TYPE, "xlink:type"},
@@ -51,14 +62,7 @@ static SPStyleProp const props[] = {
     {SP_ATTR_XLINK_ACTUATE, "xlink:actuate"},
     {SP_ATTR_TARGET, "target"},
     {SP_ATTR_INKSCAPE_GROUPMODE, "inkscape:groupmode"},
-    /* SPRoot */
-    {SP_ATTR_VERSION, "version"},
-    {SP_ATTR_WIDTH, "width"},
-    {SP_ATTR_HEIGHT, "height"},
-    {SP_ATTR_VIEWBOX, "viewBox"},
-    {SP_ATTR_PRESERVEASPECTRATIO, "preserveAspectRatio"},
-    {SP_ATTR_INKSCAPE_VERSION, "inkscape:version"},
-    {SP_ATTR_ONLOAD, "onload"},
+    {SP_ATTR_INKSCAPE_EXPANDED, "inkscape:expanded"},
     /* SPNamedView */
     {SP_ATTR_VIEWONLY, "viewonly"},
     {SP_ATTR_SHOWGUIDES, "showguides"},
@@ -76,6 +80,11 @@ static SPStyleProp const props[] = {
     {SP_ATTR_BORDERCOLOR, "bordercolor"},
     {SP_ATTR_BORDEROPACITY, "borderopacity"},
     {SP_ATTR_PAGECOLOR, "pagecolor"},
+    {SP_ATTR_FIT_MARGIN_TOP, "fit-margin-top"},
+    {SP_ATTR_FIT_MARGIN_LEFT, "fit-margin-left"},
+    {SP_ATTR_FIT_MARGIN_RIGHT, "fit-margin-right"},
+    {SP_ATTR_FIT_MARGIN_BOTTOM, "fit-margin-bottom"},
+    {SP_ATTR_INKSCAPE_PAGECHECKERBOARD, "inkscape:pagecheckerboard"},
     {SP_ATTR_INKSCAPE_PAGEOPACITY, "inkscape:pageopacity"},
     {SP_ATTR_INKSCAPE_PAGESHADOW, "inkscape:pageshadow"},
     {SP_ATTR_INKSCAPE_ZOOM, "inkscape:zoom"},
@@ -87,26 +96,33 @@ static SPStyleProp const props[] = {
     {SP_ATTR_INKSCAPE_WINDOW_Y, "inkscape:window-y"},
     {SP_ATTR_INKSCAPE_WINDOW_MAXIMIZED, "inkscape:window-maximized"},
     {SP_ATTR_INKSCAPE_SNAP_GLOBAL, "inkscape:snap-global"},
+    {SP_ATTR_INKSCAPE_SNAP_PERP, "inkscape:snap-perpendicular"},
+    {SP_ATTR_INKSCAPE_SNAP_TANG, "inkscape:snap-tangential"},
     {SP_ATTR_INKSCAPE_SNAP_BBOX, "inkscape:snap-bbox"},
-    {SP_ATTR_INKSCAPE_SNAP_NODES, "inkscape:snap-nodes"},
+    {SP_ATTR_INKSCAPE_SNAP_NODE, "inkscape:snap-nodes"},
+    {SP_ATTR_INKSCAPE_SNAP_OTHERS, "inkscape:snap-others"},
     {SP_ATTR_INKSCAPE_SNAP_FROM_GUIDE, "inkscape:snap-from-guide"},
-    {SP_ATTR_INKSCAPE_SNAP_CENTER, "inkscape:snap-center"},
-    {SP_ATTR_INKSCAPE_SNAP_GRIDS, "inkscape:snap-grids"},
-    {SP_ATTR_INKSCAPE_SNAP_TO_GUIDES, "inkscape:snap-to-guides"},
-    {SP_ATTR_INKSCAPE_SNAP_SMOOTH_NODES, "inkscape:snap-smooth-nodes"},
-    {SP_ATTR_INKSCAPE_SNAP_LINE_MIDPOINTS, "inkscape:snap-midpoints"},
-    {SP_ATTR_INKSCAPE_SNAP_OBJECT_MIDPOINTS, "inkscape:snap-object-midpoints"},
-    {SP_ATTR_INKSCAPE_SNAP_BBOX_EDGE_MIDPOINTS, "inkscape:snap-bbox-edge-midpoints"},
-	{SP_ATTR_INKSCAPE_SNAP_BBOX_MIDPOINTS, "inkscape:snap-bbox-midpoints"},
-	{SP_ATTR_INKSCAPE_SNAP_INTERS_PATHS, "inkscape:snap-intersection-paths"},
-    {SP_ATTR_INKSCAPE_OBJECT_PATHS, "inkscape:object-paths"},
-    {SP_ATTR_INKSCAPE_OBJECT_NODES, "inkscape:object-nodes"},
-    {SP_ATTR_INKSCAPE_BBOX_PATHS, "inkscape:bbox-paths"},
-    {SP_ATTR_INKSCAPE_BBOX_NODES, "inkscape:bbox-nodes"},
-    {SP_ATTR_INKSCAPE_SNAP_PAGE, "inkscape:snap-page"},
+    {SP_ATTR_INKSCAPE_SNAP_ROTATION_CENTER, "inkscape:snap-center"},
+    {SP_ATTR_INKSCAPE_SNAP_GRID, "inkscape:snap-grids"},
+    {SP_ATTR_INKSCAPE_SNAP_GUIDE, "inkscape:snap-to-guides"},
+    {SP_ATTR_INKSCAPE_SNAP_NODE_SMOOTH, "inkscape:snap-smooth-nodes"},
+    {SP_ATTR_INKSCAPE_SNAP_LINE_MIDPOINT, "inkscape:snap-midpoints"},
+    {SP_ATTR_INKSCAPE_SNAP_OBJECT_MIDPOINT, "inkscape:snap-object-midpoints"},
+    {SP_ATTR_INKSCAPE_SNAP_TEXT_BASELINE, "inkscape:snap-text-baseline"},
+    {SP_ATTR_INKSCAPE_SNAP_BBOX_EDGE_MIDPOINT, "inkscape:snap-bbox-edge-midpoints"},
+    {SP_ATTR_INKSCAPE_SNAP_BBOX_MIDPOINT, "inkscape:snap-bbox-midpoints"},
+    {SP_ATTR_INKSCAPE_SNAP_PATH_INTERSECTION, "inkscape:snap-intersection-paths"},
+    {SP_ATTR_INKSCAPE_SNAP_PATH, "inkscape:object-paths"},
+    {SP_ATTR_INKSCAPE_SNAP_PATH_CLIP, "inkscape:snap-path-clip"},
+    {SP_ATTR_INKSCAPE_SNAP_PATH_MASK, "inkscape:snap-path-mask"},
+    {SP_ATTR_INKSCAPE_SNAP_NODE_CUSP, "inkscape:object-nodes"},
+    {SP_ATTR_INKSCAPE_SNAP_BBOX_EDGE, "inkscape:bbox-paths"},
+    {SP_ATTR_INKSCAPE_SNAP_BBOX_CORNER, "inkscape:bbox-nodes"},
+    {SP_ATTR_INKSCAPE_SNAP_PAGE_BORDER, "inkscape:snap-page"},
     {SP_ATTR_INKSCAPE_CURRENT_LAYER, "inkscape:current-layer"},
-    {SP_ATTR_INKSCAPE_DOCUMENT_UNITS, "inkscape:document-units"},
-    {SP_ATTR_INKSCAPE_CONNECTOR_SPACING, "inkscape:connector-spacing"},
+    {SP_ATTR_INKSCAPE_DOCUMENT_UNITS, "inkscape:document-units"},  // This setting sets the Display units, *not* the units used in SVG
+    {SP_ATTR_INKSCAPE_LOCKGUIDES, "inkscape:lockguides"},
+    {SP_ATTR_UNITS, "units"},
     /* SPColorProfile */
     {SP_ATTR_LOCAL, "local"},
     {SP_ATTR_NAME, "name"},
@@ -114,6 +130,8 @@ static SPStyleProp const props[] = {
     /* SPGuide */
     {SP_ATTR_ORIENTATION, "orientation"},
     {SP_ATTR_POSITION, "position"},
+    {SP_ATTR_INKSCAPE_COLOR, "inkscape:color"},
+    {SP_ATTR_INKSCAPE_LOCKED, "inkscape:locked"},
     /* SPImage */
     {SP_ATTR_X, "x"},
     {SP_ATTR_Y, "y"},
@@ -122,8 +140,12 @@ static SPStyleProp const props[] = {
     {SP_ATTR_INKSCAPE_ORIGINAL_D, "inkscape:original-d"},
     /* (Note: XML representation of connectors may change in future.) */
     {SP_ATTR_CONNECTOR_TYPE, "inkscape:connector-type"},
+    {SP_ATTR_CONNECTOR_CURVATURE, "inkscape:connector-curvature"},
+    {SP_ATTR_INKSCAPE_CONNECTOR_SPACING, "inkscape:connector-spacing"},
     {SP_ATTR_CONNECTION_START, "inkscape:connection-start"},
     {SP_ATTR_CONNECTION_END, "inkscape:connection-end"},
+    {SP_ATTR_CONNECTION_START_POINT, "inkscape:connection-start-point"},
+    {SP_ATTR_CONNECTION_END_POINT, "inkscape:connection-end-point"},
     /* SPRect */
     {SP_ATTR_RX, "rx"},
     {SP_ATTR_RY, "ry"},
@@ -181,6 +203,8 @@ static SPStyleProp const props[] = {
     {SP_ATTR_DX, "dx"},
     {SP_ATTR_DY, "dy"},
     {SP_ATTR_ROTATE, "rotate"},
+    {SP_ATTR_TEXTLENGTH, "textLength"},
+    {SP_ATTR_LENGTHADJUST, "lengthAdjust"},
     {SP_ATTR_SODIPODI_ROLE, "sodipodi:role"},
     /* SPText */
     {SP_ATTR_SODIPODI_LINESPACING, "sodipodi:linespacing"},
@@ -245,9 +269,6 @@ static SPStyleProp const props[] = {
     {SP_ATTR_POINTSATY, "pointsAtY"},
     {SP_ATTR_POINTSATZ, "pointsAtZ"},
     {SP_ATTR_LIMITINGCONEANGLE, "limitingConeAngle"},
-    /*feFlood*/
-    {SP_ATTR_FLOODCOLOR, "flood-color"},
-    {SP_ATTR_FLOODOPACITY, "flood-opacity"},
     /* SPGaussianBlur */
     {SP_ATTR_STDDEVIATION, "stdDeviation"},
     /*feImage*/
@@ -272,13 +293,23 @@ static SPStyleProp const props[] = {
     {SP_ATTR_GRADIENTUNITS, "gradientUnits"},
     {SP_ATTR_GRADIENTTRANSFORM, "gradientTransform"},
     {SP_ATTR_SPREADMETHOD, "spreadMethod"},
+    {SP_ATTR_OSB_SWATCH, "osb:paint"},
     /* SPRadialGradient */
     {SP_ATTR_FX, "fx"},
     {SP_ATTR_FY, "fy"},
+    {SP_ATTR_FR, "fr"},
+    /* SPMeshPatch */
+    {SP_ATTR_TENSOR, "tensor"},
+    //{SP_ATTR_TYPE, "type"},
     /* SPPattern */
     {SP_ATTR_PATTERNUNITS, "patternUnits"},
     {SP_ATTR_PATTERNCONTENTUNITS, "patternContentUnits"},
     {SP_ATTR_PATTERNTRANSFORM, "patternTransform"},
+    /* SPHatch */
+    {SP_ATTR_HATCHUNITS, "hatchUnits"},
+    {SP_ATTR_HATCHCONTENTUNITS, "hatchContentUnits"},
+    {SP_ATTR_HATCHTRANSFORM, "hatchTransform"},
+    {SP_ATTR_PITCH, "pitch"},
     /* SPClipPath */
     {SP_ATTR_CLIPPATHUNITS, "clipPathUnits"},
     /* SPMask */
@@ -386,29 +417,78 @@ static SPStyleProp const props[] = {
     {SP_ATTR_TEXT_EXCLUDE,"inkscape:excludeShape"},
     {SP_ATTR_LAYOUT_OPTIONS,"inkscape:layoutOptions"},
 
-    /* CSS2 */
-    {SP_PROP_INKSCAPE_FONT_SPEC, "-inkscape-font-specification"},
+    /* CSS & SVG Properites */
+
+    /* Paint */
+    {SP_PROP_COLOR, "color"},
+    {SP_PROP_OPACITY, "opacity"},
+    {SP_PROP_FILL, "fill"},
+    {SP_PROP_FILL_OPACITY, "fill-opacity"},
+    {SP_PROP_FILL_RULE, "fill-rule"},
+    {SP_PROP_STROKE, "stroke"},
+    {SP_PROP_STROKE_OPACITY, "stroke-opacity"},
+    {SP_PROP_STROKE_WIDTH, "stroke-width"},
+    {SP_PROP_STROKE_LINECAP, "stroke-linecap"},
+    {SP_PROP_STROKE_LINEJOIN, "stroke-linejoin"},
+    {SP_PROP_STROKE_MITERLIMIT, "stroke-miterlimit"},
+    {SP_PROP_STROKE_DASHARRAY, "stroke-dasharray"},
+    {SP_PROP_STROKE_DASHOFFSET, "stroke-dashoffset"},
+    {SP_PROP_MARKER, "marker"},
+    {SP_PROP_MARKER_END, "marker-end"},
+    {SP_PROP_MARKER_MID, "marker-mid"},
+    {SP_PROP_MARKER_START, "marker-start"},
+    {SP_PROP_PAINT_ORDER, "paint-order" },
+    {SP_PROP_SOLID_COLOR, "solid-color"},
+    {SP_PROP_SOLID_OPACITY, "solid-opacity"},
+    {SP_PROP_VECTOR_EFFECT, "vector-effect"},
+
+    /* CSS Blending/Compositing */
+    {SP_PROP_MIX_BLEND_MODE, "mix-blend-mode"},
+    {SP_PROP_ISOLATION, "isolation"},
+
+    /* Misc. Display */
+    {SP_PROP_DISPLAY, "display"},
+    {SP_PROP_OVERFLOW, "overflow"},
+    {SP_PROP_VISIBILITY, "visibility"},
+
+    /* Clip/Mask */
+    {SP_PROP_CLIP, "clip"},
+    {SP_PROP_CLIP_PATH, "clip-path"},
+    {SP_PROP_CLIP_RULE, "clip-rule"},
+    {SP_PROP_MASK, "mask"},
+
     /* Font */
     {SP_PROP_FONT, "font"},
     {SP_PROP_FONT_FAMILY, "font-family"},
+    {SP_PROP_INKSCAPE_FONT_SPEC, "-inkscape-font-specification"},
     {SP_PROP_FONT_SIZE, "font-size"},
     {SP_PROP_FONT_SIZE_ADJUST, "font-size-adjust"},
     {SP_PROP_FONT_STRETCH, "font-stretch"},
     {SP_PROP_FONT_STYLE, "font-style"},
     {SP_PROP_FONT_VARIANT, "font-variant"},
     {SP_PROP_FONT_WEIGHT, "font-weight"},
+
+    /* Font Variants CSS 3 */
+    {SP_PROP_FONT_VARIANT_LIGATURES,  "font-variant-ligatures"},
+    {SP_PROP_FONT_VARIANT_POSITION,   "font-variant-position"},
+    {SP_PROP_FONT_VARIANT_CAPS,       "font-variant-caps"},
+    {SP_PROP_FONT_VARIANT_NUMERIC,    "font-variant-numeric"},
+    {SP_PROP_FONT_VARIANT_ALTERNATES, "font-variant-alternates"},
+    {SP_PROP_FONT_VARIANT_EAST_ASIAN, "font-variant-east-asian"},
+    {SP_PROP_FONT_FEATURE_SETTINGS,   "font-feature-settings"},
+
     /* Text */
     {SP_PROP_TEXT_INDENT, "text-indent"},
     {SP_PROP_TEXT_ALIGN, "text-align"},
-    {SP_PROP_TEXT_DECORATION, "text-decoration"},
     {SP_PROP_LINE_HEIGHT, "line-height"},
     {SP_PROP_LETTER_SPACING, "letter-spacing"},
     {SP_PROP_WORD_SPACING, "word-spacing"},
     {SP_PROP_TEXT_TRANSFORM, "text-transform"},
-    /* Text (css3) */
+
+    /* Text (CSS3) */
     {SP_PROP_DIRECTION, "direction"},
-    {SP_PROP_BLOCK_PROGRESSION, "block-progression"},
     {SP_PROP_WRITING_MODE, "writing-mode"},
+    {SP_PROP_TEXT_ORIENTATION, "text-orientation"},
     {SP_PROP_UNICODE_BIDI, "unicode-bidi"},
     {SP_PROP_ALIGNMENT_BASELINE, "alignment-baseline"},
     {SP_PROP_BASELINE_SHIFT, "baseline-shift"},
@@ -417,57 +497,52 @@ static SPStyleProp const props[] = {
     {SP_PROP_GLYPH_ORIENTATION_VERTICAL, "glyph-orientation-vertical"},
     {SP_PROP_KERNING, "kerning"},
     {SP_PROP_TEXT_ANCHOR, "text-anchor"},
-    /* Misc */
-    {SP_PROP_CLIP, "clip"},
-    {SP_PROP_COLOR, "color"},
-    {SP_PROP_CURSOR, "cursor"},
-    {SP_PROP_DISPLAY, "display"},
-    {SP_PROP_OVERFLOW, "overflow"},
-    {SP_PROP_VISIBILITY, "visibility"},
-    /* SVG */
-    /* Clip/Mask */
-    {SP_PROP_CLIP_PATH, "clip-path"},
-    {SP_PROP_CLIP_RULE, "clip-rule"},
-    {SP_PROP_MASK, "mask"},
-    {SP_PROP_OPACITY, "opacity"},
+    {SP_PROP_WHITE_SPACE, "white-space"},
+
+    /* SVG 2 Text Wrapping */
+    {SP_PROP_SHAPE_INSIDE,  "shape-inside"},
+    {SP_PROP_SHAPE_OUTSIDE, "shape-outside"},
+    {SP_PROP_SHAPE_PADDING, "shape-padding"},
+    {SP_PROP_SHAPE_MARGIN,  "shape-margin"},
+
+    /* Text Decoration */
+    {SP_PROP_TEXT_DECORATION,       "text-decoration"},  // CSS 2/CSS3-Shorthand
+    {SP_PROP_TEXT_DECORATION_LINE,  "text-decoration-line"},
+    {SP_PROP_TEXT_DECORATION_STYLE, "text-decoration-style"},
+    {SP_PROP_TEXT_DECORATION_COLOR, "text-decoration-color"},
+    {SP_PROP_TEXT_DECORATION_FILL,  "text-decoration-fill"},
+    {SP_PROP_TEXT_DECORATION_STROKE,"text-decoration-stroke"},
+
     /* Filter */
     {SP_PROP_ENABLE_BACKGROUND, "enable-background"},
     {SP_PROP_FILTER, "filter"},
     {SP_PROP_FLOOD_COLOR, "flood-color"},
     {SP_PROP_FLOOD_OPACITY, "flood-opacity"},
     {SP_PROP_LIGHTING_COLOR, "lighting-color"},
+
     /* Gradient */
     {SP_PROP_STOP_COLOR, "stop-color"},
     {SP_PROP_STOP_OPACITY, "stop-opacity"},
-    /* Interactivity */
-    {SP_PROP_POINTER_EVENTS, "pointer-events"},
-    /* Paint */
+    {SP_PROP_STOP_PATH, "path"},
+
+    /* Rendering */
     {SP_PROP_COLOR_INTERPOLATION, "color-interpolation"},
     {SP_PROP_COLOR_INTERPOLATION_FILTERS, "color-interpolation-filters"},
     {SP_PROP_COLOR_PROFILE, "color-profile"},
     {SP_PROP_COLOR_RENDERING, "color-rendering"},
-    {SP_PROP_FILL, "fill"},
-    {SP_PROP_FILL_OPACITY, "fill-opacity"},
-    {SP_PROP_FILL_RULE, "fill-rule"},
     {SP_PROP_IMAGE_RENDERING, "image-rendering"},
-    {SP_PROP_MARKER, "marker"},
-    {SP_PROP_MARKER_END, "marker-end"},
-    {SP_PROP_MARKER_MID, "marker-mid"},
-    {SP_PROP_MARKER_START, "marker-start"},
     {SP_PROP_SHAPE_RENDERING, "shape-rendering"},
-    {SP_PROP_STROKE, "stroke"},
-    {SP_PROP_STROKE_DASHARRAY, "stroke-dasharray"},
-    {SP_PROP_STROKE_DASHOFFSET, "stroke-dashoffset"},
-    {SP_PROP_STROKE_LINECAP, "stroke-linecap"},
-    {SP_PROP_STROKE_LINEJOIN, "stroke-linejoin"},
-    {SP_PROP_STROKE_MITERLIMIT, "stroke-miterlimit"},
-    {SP_PROP_STROKE_OPACITY, "stroke-opacity"},
-    {SP_PROP_STROKE_WIDTH, "stroke-width"},
     {SP_PROP_TEXT_RENDERING, "text-rendering"},
+
+    /* Interactivity */
+    {SP_PROP_POINTER_EVENTS, "pointer-events"},
+    {SP_PROP_CURSOR, "cursor"},
+
     /* Conditional */
     {SP_PROP_SYSTEM_LANGUAGE, "systemLanguage"},
     {SP_PROP_REQUIRED_FEATURES, "requiredFeatures"},
     {SP_PROP_REQUIRED_EXTENSIONS, "requiredExtensions"},
+
     /* LivePathEffect */
     {SP_PROP_PATH_EFFECT, "effect"},
 };
@@ -478,20 +553,15 @@ static SPStyleProp const props[] = {
 unsigned
 sp_attribute_lookup(gchar const *key)
 {
-    static GHashTable *propdict = NULL;
-
-    if (!propdict) {
-        unsigned int i;
-        propdict = g_hash_table_new(g_str_hash, g_str_equal);
-        for (i = 1; i < n_attrs; i++) {
-            g_assert(props[i].code == static_cast< gint >(i) );
-            g_hash_table_insert(propdict,
-                                const_cast<void *>(static_cast<void const *>(props[i].name)),
-                                GINT_TO_POINTER(props[i].code));
-        }
+    for (unsigned int i = 1; i < n_attrs; i++) {
+        g_assert(props[i].code == static_cast< gint >(i) );
+        // If this g_assert fails, then the sort order of SPAttributeEnum does not match the order in props[]!
+        if(g_str_equal(const_cast<void *>(static_cast<void const *>(props[i].name)), key))
+            return GPOINTER_TO_UINT(GINT_TO_POINTER(props[i].code));
     }
-
-    return GPOINTER_TO_UINT(g_hash_table_lookup(propdict, key));
+    // std::cerr << "sp_attribute_lookup: invalid attribute: "
+    //           << (key?key:"Null") << std::endl;
+    return SP_ATTR_INVALID;
 }
 
 unsigned char const *
@@ -514,4 +584,4 @@ sp_attribute_name(unsigned int id)
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8 :
