@@ -432,6 +432,17 @@ void Preferences::setBool(Glib::ustring const &pref_path, bool value)
     _setRawValue(pref_path, ( value ? "1" : "0" ));
 }
 
+/**
+ * Set an point attribute of a preference.
+ *
+ * @param pref_path Path of the preference to modify.
+ * @param value The new value of the pref attribute.
+ */
+void Preferences::setPoint(Glib::ustring const &pref_path, Geom::Point value)
+{
+    _setRawValue(pref_path, Glib::ustring::compose("%1",value[Geom::X]) + "," + Glib::ustring::compose("%1",value[Geom::Y]));
+}
+
 /**
  * Set an integer attribute of a preference.
  *
@@ -680,7 +691,9 @@ Inkscape::XML::Node *Preferences::_getNode(Glib::ustring const &pref_key, bool c
             }
 
             for (child = node->firstChild(); child; child = child->next()) {
-                if (!strcmp(splits[part_i], child->attribute("id"))) {
+                // To do: figure out why a comment in the XML file causes a crash if we don't
+                // check for child->attribute("id").
+                if (child->attribute("id") && !strcmp(splits[part_i], child->attribute("id"))) {
                     break;
                 }
             }
@@ -759,6 +772,16 @@ bool Preferences::_extractBool(Entry const &v)
     }
 }
 
+Geom::Point Preferences::_extractPoint(Entry const &v)
+{
+    gchar const *s = static_cast<gchar const *>(v._value);
+    gchar ** strarray = g_strsplit(s, ",", 2);
+    double newx = atoi(strarray[0]);
+    double newy = atoi(strarray[1]);
+    g_strfreev (strarray);
+    return Geom::Point(newx, newy);
+}
+
 int Preferences::_extractInt(Entry const &v)
 {
     gchar const *s = static_cast<gchar const *>(v._value);
