@@ -4,7 +4,8 @@
 enum SPGradientSpread {
     SP_GRADIENT_SPREAD_PAD,
     SP_GRADIENT_SPREAD_REFLECT,
-    SP_GRADIENT_SPREAD_REPEAT
+    SP_GRADIENT_SPREAD_REPEAT,
+    SP_GRADIENT_SPREAD_UNDEFINED = INT_MAX
 };
 
 
@@ -19,4 +20,4 @@ enum SPGradientSpread {
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
