@@ -122,7 +122,7 @@ AboutBox::AboutBox() : Gtk::Dialog(_("About Inkscape")) {
 
     Gtk::Label *link = new Gtk::Label();
     const gchar *website_link =
-        "<a href=\"https://www.inkscape.org\"> https://www.inkscape.org</a>";
+        "<a href=\"https://www.inkscape.org\">https://www.inkscape.org</a>";
 
     link->set_markup(website_link);
     link->set_alignment(Gtk::ALIGN_END);
@@ -166,16 +166,16 @@ Gtk::Widget *build_splash_widget() {
        the `screens' directory.  Thus the translation of "about.svg" should be
        the filename of its translated version, e.g. about.zh.svg for Chinese.
 
-       N.B. about.svg changes once per release.  (We should probably rename
-       the original to about-0.40.svg etc. as soon as we have a translation.
-       If we do so, then add an item to release-checklist saying that the
-       string here should be changed.) */
+       Please don't translate the filename unless the translated picture exists. */
 
     // FIXME? INKSCAPE_SCREENSDIR and "about.svg" are in UTF-8, not the
     // native filename encoding... and the filename passed to sp_document_new
     // should be in UTF-*8..
 
     char *about=g_build_filename(INKSCAPE_SCREENSDIR, _("about.svg"), NULL);
+    if (!g_file_test (about, G_FILE_TEST_EXISTS)) {
+        about=g_build_filename(INKSCAPE_SCREENSDIR, "about.svg", NULL);
+    }
     SPDocument *doc=SPDocument::createNewDoc (about, TRUE);
     g_free(about);
     g_return_val_if_fail(doc != NULL, NULL);
@@ -366,6 +366,7 @@ void AboutBox::initStrings() {
 "Dmitry G. Mastrukov\n"
 "David Mathog\n"
 "Matiphas\n"
+"Patrick McDermott\n"
 "Michael Meeks\n"
 "Federico Mena\n"
 "MenTaLguY\n"
