@@ -1,5 +1,3 @@
-#define __VANISHING_POINT_C__
-
 /*
  * Vanishing point for 3D perspectives
  *
@@ -7,6 +5,8 @@
  *   bulia byak <buliabyak@users.sf.net>
  *   Johan Engelen <j.b.c.engelen@ewi.utwente.nl>
  *   Maximilian Albert <Anhalter42@gmx.de>
+ *   Abhishek Sharma
+ *   Jon A. Cruz <jon@joncruz.org>
  *
  * Copyright (C) 2005-2007 authors
  *
@@ -16,24 +16,33 @@
 #include <glibmm/i18n.h>
 
 #include "vanishing-point.h"
-#include "desktop-handles.h"
+
 #include "desktop.h"
-#include "event-context.h"
+#include "display/sp-canvas-item.h"
+#include "display/sp-ctrlline.h"
+#include "ui/tools/tool-base.h"
 #include "xml/repr.h"
 #include "perspective-line.h"
+#include "ui/shape-editor.h"
+#include "snap.h"
+#include "sp-namedview.h"
+#include "ui/control-manager.h"
+#include "document-undo.h"
+#include "verbs.h"
 
-#include "knotholder.h" // FIXME: can we avoid direct access to knotholder_update_knots?
+using Inkscape::CTLINE_PRIMARY;
+using Inkscape::CTLINE_SECONDARY;
+using Inkscape::CTLINE_TERTIARY;
+using Inkscape::CTRL_TYPE_ANCHOR;
+using Inkscape::ControlManager;
+using Inkscape::CtrlLineType;
+using Inkscape::DocumentUndo;
 
 namespace Box3D {
 
 #define VP_KNOT_COLOR_NORMAL 0xffffff00
 #define VP_KNOT_COLOR_SELECTED 0x0000ff00
 
-#define VP_LINE_COLOR_FILL 0x0000ff7f
-#define VP_LINE_COLOR_STROKE_X 0xff00007f
-#define VP_LINE_COLOR_STROKE_Y 0x0000ff7f
-#define VP_LINE_COLOR_STROKE_Z 0xffff007f
-
 // screen pixels between knots when they snap:
 #define SNAP_DIST 5
 
@@ -77,15 +86,15 @@ have_VPs_of_same_perspective (VPDragger *dr1, VPDragger *dr2)
 }
 
 static void
-vp_knot_moved_handler (SPKnot */*knot*/, NR::Point const *ppointer, guint state, gpointer data)
+vp_knot_moved_handler (SPKnot *knot, Geom::Point const &ppointer, guint state, gpointer data)
 {
     VPDragger *dragger = (VPDragger *) data;
     VPDrag *drag = dragger->parent;
 
-    NR::Point p = *ppointer;
+    Geom::Point p = ppointer;
 
     // FIXME: take from prefs
-    double snap_dist = SNAP_DIST / inkscape_active_desktop()->current_zoom();
+    double snap_dist = SNAP_DIST / SP_ACTIVE_DESKTOP->current_zoom();
 
     /*
      * We use dragging_started to indicate if we have already checked for the need to split Draggers up.
@@ -101,10 +110,10 @@ vp_knot_moved_handler (SPKnot */*knot*/, NR::Point const *ppointer, guint state,
             for (std::set<VanishingPoint*, less_ptr>::iterator vp = sel_vps.begin(); vp != sel_vps.end(); ++vp) {
                 // for each VP that has selected boxes:
                 Persp3D *old_persp = (*vp)->get_perspective();
-                sel_boxes = (*vp)->selectedBoxes(sp_desktop_selection(inkscape_active_desktop()));
+                sel_boxes = (*vp)->selectedBoxes(SP_ACTIVE_DESKTOP->getSelection());
 
                 // we create a new perspective ...
-                Persp3D *new_persp = persp3d_create_xml_element (dragger->parent->document, old_persp);
+                Persp3D *new_persp = persp3d_create_xml_element (dragger->parent->document, old_persp->perspective_impl);
 
                 /* ... unlink the boxes from the old one and
                    FIXME: We need to unlink the _un_selected boxes of each VP so that
@@ -120,17 +129,17 @@ vp_knot_moved_handler (SPKnot */*knot*/, NR::Point const *ppointer, guint state,
             }
             // FIXME: Do we need to create a new dragger as well?
             dragger->updateZOrders ();
-            sp_document_done (sp_desktop_document (inkscape_active_desktop()), SP_VERB_CONTEXT_3DBOX,
-                              _("Split vanishing points"));
+            DocumentUndo::done(SP_ACTIVE_DESKTOP->getDocument(), SP_VERB_CONTEXT_3DBOX,
+                   _("Split vanishing points"));
             return;
         }
     }
 
     if (!(state & GDK_SHIFT_MASK)) {
         // without Shift; see if we need to snap to another dragger
-        for (GList *di = dragger->parent->draggers; di != NULL; di = di->next) {
-            VPDragger *d_new = (VPDragger *) di->data;
-            if ((d_new != dragger) && (NR::L2 (d_new->point - p) < snap_dist)) {
+        for (std::vector<VPDragger *>::const_iterator di = dragger->parent->draggers.begin(); di != dragger->parent->draggers.end(); ++di) {
+            VPDragger *d_new = *di; 
+            if ((d_new != dragger) && (Geom::L2 (d_new->point - p) < snap_dist)) {
                 if (have_VPs_of_same_perspective (dragger, d_new)) {
                     // this would result in degenerate boxes, which we disallow for the time being
                     continue;
@@ -146,7 +155,7 @@ vp_knot_moved_handler (SPKnot */*knot*/, NR::Point const *ppointer, guint state,
                 d_new->vps.merge(dragger->vps);
 
                 // ... delete old dragger ...
-                drag->draggers = g_list_remove (drag->draggers, dragger);
+                drag->draggers.erase(std::remove(drag->draggers.begin(), drag->draggers.end(), dragger),drag->draggers.end());
                 delete dragger;
                 dragger = NULL;
 
@@ -165,16 +174,28 @@ vp_knot_moved_handler (SPKnot */*knot*/, NR::Point const *ppointer, guint state,
                 //       deleted according to changes in the svg representation, not based on any user input
                 //       as is currently the case.
 
-                sp_document_done (sp_desktop_document (inkscape_active_desktop()), SP_VERB_CONTEXT_3DBOX,
-                                  _("Merge vanishing points"));
+                DocumentUndo::done(SP_ACTIVE_DESKTOP->getDocument(), SP_VERB_CONTEXT_3DBOX,
+                   _("Merge vanishing points"));
 
                 return;
             }
         }
     }
 
+    // We didn't hit the return statement above, so we didn't snap to another dragger. Therefore we'll now try a regular snap
+    // Regardless of the status of the SHIFT key, we will try to snap; Here SHIFT does not disable snapping, as the shift key
+    // has a different purpose in this context (see above)
+    SPDesktop *desktop = SP_ACTIVE_DESKTOP;
+    SnapManager &m = desktop->namedview->snap_manager;
+    m.setup(desktop);
+    Inkscape::SnappedPoint s = m.freeSnap(Inkscape::SnapCandidatePoint(p, Inkscape::SNAPSOURCE_OTHER_HANDLE));
+    m.unSetup();
+    if (s.getSnapped()) {
+        p = s.getPoint();
+        knot->moveto(p);
+    }
 
-    dragger->point = p; // FIXME: Brauchen wir dragger->point überhaupt?
+    dragger->point = p; // FIXME: Is dragger->point being used at all?
 
     dragger->updateVPs(p);
     dragger->updateBoxDisplays();
@@ -186,7 +207,7 @@ vp_knot_moved_handler (SPKnot */*knot*/, NR::Point const *ppointer, guint state,
     dragger->dragging_started = true;
 }
 
-void
+static void
 vp_knot_grabbed_handler (SPKnot */*knot*/, unsigned int /*state*/, gpointer data)
 {
     VPDragger *dragger = (VPDragger *) data;
@@ -221,8 +242,8 @@ vp_knot_ungrabbed_handler (SPKnot *knot, guint /*state*/, gpointer data)
     // TODO: Undo machinery!!
     g_return_if_fail (dragger->parent);
     g_return_if_fail (dragger->parent->document);
-    sp_document_done(dragger->parent->document, SP_VERB_CONTEXT_3DBOX,
-                     _("3D box: Move vanishing point"));
+    DocumentUndo::done(dragger->parent->document, SP_VERB_CONTEXT_3DBOX,
+               _("3D box: Move vanishing point"));
 }
 
 unsigned int VanishingPoint::global_counter = 0;
@@ -231,48 +252,49 @@ unsigned int VanishingPoint::global_counter = 0;
 void
 VanishingPoint::set_pos(Proj::Pt2 const &pt) {
     g_return_if_fail (_persp);
-    _persp->tmat.set_image_pt (_axis, pt);
+    _persp->perspective_impl->tmat.set_image_pt (_axis, pt);
 }
 
 std::list<SPBox3D *>
 VanishingPoint::selectedBoxes(Inkscape::Selection *sel) {
     std::list<SPBox3D *> sel_boxes;
-    for (GSList const* i = sel->itemList(); i != NULL; i = i->next) {
-        if (!SP_IS_BOX3D(i->data))
-            continue;
-        SPBox3D *box = SP_BOX3D(i->data);
-        if (this->hasBox(box)) {
-            sel_boxes.push_back (box);
+    std::vector<SPItem*> itemlist=sel->itemList();
+    for (std::vector<SPItem*>::const_iterator i=itemlist.begin();i!=itemlist.end();++i) {
+        SPItem *item = *i;
+        SPBox3D *box = dynamic_cast<SPBox3D *>(item);
+        if (box && this->hasBox(box)) {
+            sel_boxes.push_back(box);
         }
     }
     return sel_boxes;
 }
 
-VPDragger::VPDragger(VPDrag *parent, NR::Point p, VanishingPoint &vp)
+VPDragger::VPDragger(VPDrag *parent, Geom::Point p, VanishingPoint &vp) :
+    parent(parent),
+    knot(NULL),
+    point(p),
+    point_original(p),
+    dragging_started(false),
+    vps()
 {
-    this->parent = parent;
-
-    this->point = p;
-    this->point_original = p;
-
-    this->dragging_started = false;
-
     if (vp.is_finite()) {
         // create the knot
-        this->knot = sp_knot_new (inkscape_active_desktop(), NULL);
+        this->knot = new SPKnot(SP_ACTIVE_DESKTOP, NULL);
         this->knot->setMode(SP_KNOT_MODE_XOR);
         this->knot->setFill(VP_KNOT_COLOR_NORMAL, VP_KNOT_COLOR_NORMAL, VP_KNOT_COLOR_NORMAL);
         this->knot->setStroke(0x000000ff, 0x000000ff, 0x000000ff);
-        sp_knot_update_ctrl(this->knot);
+        this->knot->updateCtrl();
+        knot->item->ctrlType = CTRL_TYPE_ANCHOR;
+        ControlManager::getManager().track(knot->item);
 
         // move knot to the given point
-        sp_knot_set_position (this->knot, &this->point, SP_KNOT_STATE_NORMAL);
-        sp_knot_show (this->knot);
+        this->knot->setPosition(this->point, SP_KNOT_STATE_NORMAL);
+        this->knot->show();
 
         // connect knot's signals
-        g_signal_connect (G_OBJECT (this->knot), "moved", G_CALLBACK (vp_knot_moved_handler), this);
-        g_signal_connect (G_OBJECT (this->knot), "grabbed", G_CALLBACK (vp_knot_grabbed_handler), this);
-        g_signal_connect (G_OBJECT (this->knot), "ungrabbed", G_CALLBACK (vp_knot_ungrabbed_handler), this);
+        this->_moved_connection = this->knot->moved_signal.connect(sigc::bind(sigc::ptr_fun(vp_knot_moved_handler), this));
+        this->_grabbed_connection = this->knot->grabbed_signal.connect(sigc::bind(sigc::ptr_fun(vp_knot_grabbed_handler), this));
+        this->_ungrabbed_connection = this->knot->ungrabbed_signal.connect(sigc::bind(sigc::ptr_fun(vp_knot_ungrabbed_handler), this));
 
         // add the initial VP (which may be NULL!)
         this->addVP (vp);
@@ -282,11 +304,12 @@ VPDragger::VPDragger(VPDrag *parent, NR::Point p, VanishingPoint &vp)
 VPDragger::~VPDragger()
 {
     // disconnect signals
-    g_signal_handlers_disconnect_by_func(G_OBJECT(this->knot), (gpointer) G_CALLBACK (vp_knot_moved_handler), this);
-    g_signal_handlers_disconnect_by_func(G_OBJECT(this->knot), (gpointer) G_CALLBACK (vp_knot_grabbed_handler), this);
-    g_signal_handlers_disconnect_by_func(G_OBJECT(this->knot), (gpointer) G_CALLBACK (vp_knot_ungrabbed_handler), this);
+    this->_moved_connection.disconnect();
+    this->_grabbed_connection.disconnect();
+    this->_ungrabbed_connection.disconnect();
+
     /* unref should call destroy */
-    g_object_unref (G_OBJECT (this->knot));
+    knot_unref(this->knot);
 }
 
 /**
@@ -373,14 +396,16 @@ VPDragger::VPsOfSelectedBoxes() {
     std::set<VanishingPoint*, less_ptr> sel_vps;
     VanishingPoint *vp;
     // FIXME: Should we take the selection from the parent VPDrag? I guess it shouldn't make a difference.
-    Inkscape::Selection *sel = sp_desktop_selection(inkscape_active_desktop());
-    for (GSList const* i = sel->itemList(); i != NULL; i = i->next) {
-        if (!SP_IS_BOX3D(i->data))
-            continue;
-        SPBox3D *box = SP_BOX3D(i->data);
-        vp = this->findVPWithBox(box);
-        if (vp) {
-            sel_vps.insert (vp);
+    Inkscape::Selection *sel = SP_ACTIVE_DESKTOP->getSelection();
+    std::vector<SPItem*> itemlist=sel->itemList();
+    for (std::vector<SPItem*>::const_iterator i=itemlist.begin();i!=itemlist.end();++i) {
+        SPItem *item = *i;
+        SPBox3D *box = dynamic_cast<SPBox3D *>(item);
+        if (box) {
+            vp = this->findVPWithBox(box);
+            if (vp) {
+                sel_vps.insert (vp);
+            }
         }
     }
     return sel_vps;
@@ -440,7 +465,7 @@ VPDragger::updateBoxDisplays ()
 }
 
 void
-VPDragger::updateVPs (NR::Point const &pt)
+VPDragger::updateVPs (Geom::Point const &pt)
 {
     for (std::list<VanishingPoint>::iterator i = this->vps.begin(); i != this->vps.end(); ++i) {
         (*i).set_pos (pt);
@@ -457,7 +482,7 @@ VPDragger::updateZOrders ()
 
 void
 VPDragger::printVPs() {
-    g_print ("VPDragger at position (%f, %f):\n", point[NR::X], point[NR::Y]);
+    g_print ("VPDragger at position (%f, %f):\n", point[Geom::X], point[Geom::Y]);
     for (std::list<VanishingPoint>::iterator i = this->vps.begin(); i != this->vps.end(); ++i) {
         g_print ("    VP %s\n", (*i).axisString());
     }
@@ -466,10 +491,8 @@ VPDragger::printVPs() {
 VPDrag::VPDrag (SPDocument *document)
 {
     this->document = document;
-    this->selection = sp_desktop_selection(inkscape_active_desktop());
+    this->selection = SP_ACTIVE_DESKTOP->getSelection();
 
-    this->draggers = NULL;
-    this->lines = NULL;
     this->show_lines = true;
     this->front_or_rear_lines = 0x1;
 
@@ -496,17 +519,15 @@ VPDrag::~VPDrag()
     this->sel_changed_connection.disconnect();
     this->sel_modified_connection.disconnect();
 
-    for (GList *l = this->draggers; l != NULL; l = l->next) {
-        delete ((VPDragger *) l->data);
+    for (std::vector<VPDragger *>::const_iterator i = this->draggers.begin(); i != this->draggers.end(); ++i) {
+        delete (*i);
     }
-    g_list_free (this->draggers);
-    this->draggers = NULL;
+    this->draggers.clear();
 
-    for (GSList const *i = this->lines; i != NULL; i = i->next) {
-        gtk_object_destroy( GTK_OBJECT (i->data));
+    for (std::vector<SPCtrlLine *>::const_iterator i = this->lines.begin(); i != this->lines.end(); ++i) {
+        sp_canvas_item_destroy(SP_CANVAS_ITEM(*i));
     }
-    g_slist_free (this->lines);
-    this->lines = NULL;
+    this->lines.clear(); 
 }
 
 /**
@@ -515,8 +536,8 @@ VPDrag::~VPDrag()
 VPDragger *
 VPDrag::getDraggerFor (VanishingPoint const &vp)
 {
-    for (GList const* i = this->draggers; i != NULL; i = i->next) {
-        VPDragger *dragger = (VPDragger *) i->data;
+    for (std::vector<VPDragger *>::const_iterator i = this->draggers.begin(); i != this->draggers.end(); ++i) {
+        VPDragger *dragger = *i;
         for (std::list<VanishingPoint>::iterator j = dragger->vps.begin(); j != dragger->vps.end(); ++j) {
             // TODO: Should we compare the pointers or the VPs themselves!?!?!?!
             if (*j == vp) {
@@ -531,8 +552,8 @@ void
 VPDrag::printDraggers ()
 {
     g_print ("=== VPDrag info: =================================\n");
-    for (GList const* i = this->draggers; i != NULL; i = i->next) {
-        ((VPDragger *) i->data)->printVPs();
+    for (std::vector<VPDragger *>::const_iterator i = this->draggers.begin(); i != this->draggers.end(); ++i) {
+        (*i)->printVPs();
         g_print ("========\n");
     }
     g_print ("=================================================\n");
@@ -547,23 +568,23 @@ VPDrag::updateDraggers ()
     if (this->dragging)
         return;
     // delete old draggers
-    for (GList const* i = this->draggers; i != NULL; i = i->next) {
-        delete ((VPDragger *) i->data);
+    for (std::vector<VPDragger *>::const_iterator i = this->draggers.begin(); i != this->draggers.end(); ++i) {
+        delete (*i);
     }
-    g_list_free (this->draggers);
-    this->draggers = NULL;
+    this->draggers.clear();
 
     g_return_if_fail (this->selection != NULL);
 
-    for (GSList const* i = this->selection->itemList(); i != NULL; i = i->next) {
-        SPItem *item = SP_ITEM(i->data);
-        if (!SP_IS_BOX3D (item)) continue;
-        SPBox3D *box = SP_BOX3D (item);
-
-        VanishingPoint vp;
-        for (int i = 0; i < 3; ++i) {
-            vp.set(box3d_get_perspective(box), Proj::axes[i]);
-            addDragger (vp);
+    std::vector<SPItem*> itemlist=this->selection->itemList();
+    for (std::vector<SPItem*>::const_iterator i=itemlist.begin();i!=itemlist.end();++i) {
+        SPItem *item = *i;
+        SPBox3D *box = dynamic_cast<SPBox3D *>(item);
+        if (box) {
+            VanishingPoint vp;
+            for (int i = 0; i < 3; ++i) {
+                vp.set(box3d_get_perspective(box), Proj::axes[i]);
+                addDragger (vp);
+            }
         }
     }
 }
@@ -576,24 +597,25 @@ void
 VPDrag::updateLines ()
 {
     // delete old lines
-    for (GSList const *i = this->lines; i != NULL; i = i->next) {
-        gtk_object_destroy( GTK_OBJECT (i->data));
+    for (std::vector<SPCtrlLine *>::const_iterator i = this->lines.begin(); i != this->lines.end(); ++i) {
+        sp_canvas_item_destroy(SP_CANVAS_ITEM(*i));
     }
-    g_slist_free (this->lines);
-    this->lines = NULL;
+    this->lines.clear();
 
     // do nothing if perspective lines are currently disabled
     if (this->show_lines == 0) return;
 
     g_return_if_fail (this->selection != NULL);
 
-    for (GSList const* i = this->selection->itemList(); i != NULL; i = i->next) {
-        if (!SP_IS_BOX3D(i->data)) continue;
-        SPBox3D *box = SP_BOX3D (i->data);
-
-        this->drawLinesForFace (box, Proj::X);
-        this->drawLinesForFace (box, Proj::Y);
-        this->drawLinesForFace (box, Proj::Z);
+    std::vector<SPItem*> itemlist=this->selection->itemList();
+    for (std::vector<SPItem*>::const_iterator i=itemlist.begin();i!=itemlist.end();++i) {
+        SPItem *item = *i;
+        SPBox3D *box = dynamic_cast<SPBox3D *>(item);
+        if (box) {
+            this->drawLinesForFace (box, Proj::X);
+            this->drawLinesForFace (box, Proj::Y);
+            this->drawLinesForFace (box, Proj::Z);
+        }
     }
 }
 
@@ -601,29 +623,29 @@ void
 VPDrag::updateBoxHandles ()
 {
     // FIXME: Is there a way to update the knots without accessing the
-    //        (previously) statically linked function knotholder_update_knots?
+    //        (previously) statically linked function KnotHolder::update_knots?
 
-    GSList *sel = (GSList *) selection->itemList();
-    if (!sel)
+    std::vector<SPItem*> sel = selection->itemList();
+    if (sel.empty())
         return; // no selection
 
-    if (g_slist_length (sel) > 1) {
+    if (sel.size() > 1) {
         // Currently we only show handles if a single box is selected
         return;
     }
 
-    SPEventContext *ec = inkscape_active_event_context();
+    Inkscape::UI::Tools::ToolBase *ec = INKSCAPE.active_event_context();
     g_assert (ec != NULL);
-    if (ec->shape_knot_holder != NULL) {
-        knotholder_update_knots(ec->shape_knot_holder, (SPItem *) sel->data);
+    if (ec->shape_editor != NULL) {
+        ec->shape_editor->update_knotholder();
     }
 }
 
 void
 VPDrag::updateBoxReprs ()
 {
-    for (GList *i = this->draggers; i != NULL; i = i->next) {
-        VPDragger *dragger = (VPDragger *) i->data;
+    for (std::vector<VPDragger *>::const_iterator i = this->draggers.begin(); i != this->draggers.end(); ++i) {
+        VPDragger *dragger = *i;
         for (std::list<VanishingPoint>::iterator i = dragger->vps.begin(); i != dragger->vps.end(); ++i) {
             (*i).updateBoxReprs();
         }
@@ -633,8 +655,8 @@ VPDrag::updateBoxReprs ()
 void
 VPDrag::updateBoxDisplays ()
 {
-    for (GList *i = this->draggers; i != NULL; i = i->next) {
-        VPDragger *dragger = (VPDragger *) i->data;
+    for (std::vector<VPDragger *>::const_iterator i = this->draggers.begin(); i != this->draggers.end(); ++i) {
+        VPDragger *dragger = *i;
         for (std::list<VanishingPoint>::iterator i = dragger->vps.begin(); i != dragger->vps.end(); ++i) {
             (*i).updateBoxDisplays();
         }
@@ -645,41 +667,47 @@ VPDrag::updateBoxDisplays ()
 /**
  * Depending on the value of all_lines, draw the front and/or rear perspective lines starting from the given corners.
  */
-void
-VPDrag::drawLinesForFace (const SPBox3D *box, Proj::Axis axis) //, guint corner1, guint corner2, guint corner3, guint corner4)
+void VPDrag::drawLinesForFace(const SPBox3D *box, Proj::Axis axis) //, guint corner1, guint corner2, guint corner3, guint corner4)
 {
-    guint color;
+    CtrlLineType type = CTLINE_PRIMARY;
     switch (axis) {
         // TODO: Make color selectable by user
-        case Proj::X: color = VP_LINE_COLOR_STROKE_X; break;
-        case Proj::Y: color = VP_LINE_COLOR_STROKE_Y; break;
-        case Proj::Z: color = VP_LINE_COLOR_STROKE_Z; break;
-        default: g_assert_not_reached();
+        case Proj::X:
+            type = CTLINE_SECONDARY;
+            break;
+        case Proj::Y:
+            type = CTLINE_PRIMARY;
+            break;
+        case Proj::Z:
+            type = CTLINE_TERTIARY;
+            break;
+        default:
+            g_assert_not_reached();
     }
 
-    NR::Point corner1, corner2, corner3, corner4;
+    Geom::Point corner1, corner2, corner3, corner4;
     box3d_corners_for_PLs (box, axis, corner1, corner2, corner3, corner4);
 
     g_return_if_fail (box3d_get_perspective(box));
     Proj::Pt2 vp = persp3d_get_VP (box3d_get_perspective(box), axis);
     if (vp.is_finite()) {
         // draw perspective lines for finite VPs
-        NR::Point pt = vp.affine();
+        Geom::Point pt = vp.affine();
         if (this->front_or_rear_lines & 0x1) {
             // draw 'front' perspective lines
-            this->addLine (corner1, pt, color);
-            this->addLine (corner2, pt, color);
+            this->addLine(corner1, pt, type);
+            this->addLine(corner2, pt, type);
         }
         if (this->front_or_rear_lines & 0x2) {
             // draw 'rear' perspective lines
-            this->addLine (corner3, pt, color);
-            this->addLine (corner4, pt, color);
+            this->addLine(corner3, pt, type);
+            this->addLine(corner4, pt, type);
         }
     } else {
         // draw perspective lines for infinite VPs
-        NR::Maybe<NR::Point> pt1, pt2, pt3, pt4;
+        boost::optional<Geom::Point> pt1, pt2, pt3, pt4;
         Persp3D *persp = box3d_get_perspective(box);
-        SPDesktop *desktop = inkscape_active_desktop (); // FIXME: Store the desktop in VPDrag
+        SPDesktop *desktop = SP_ACTIVE_DESKTOP; // FIXME: Store the desktop in VPDrag
         Box3D::PerspectiveLine pl (corner1, axis, persp);
         pt1 = pl.intersection_with_viewbox(desktop);
 
@@ -698,13 +726,13 @@ VPDrag::drawLinesForFace (const SPBox3D *box, Proj::Axis axis) //, guint corner1
         }
         if (this->front_or_rear_lines & 0x1) {
             // draw 'front' perspective lines
-            this->addLine (corner1, *pt1, color);
-            this->addLine (corner2, *pt2, color);
+            this->addLine(corner1, *pt1, type);
+            this->addLine(corner2, *pt2, type);
         }
         if (this->front_or_rear_lines & 0x2) {
             // draw 'rear' perspective lines
-            this->addLine (corner3, *pt3, color);
-            this->addLine (corner4, *pt4, color);
+            this->addLine(corner3, *pt3, type);
+            this->addLine(corner4, *pt4, type);
         }
     }
 }
@@ -721,11 +749,11 @@ VPDrag::addDragger (VanishingPoint &vp)
         // don't create draggers for infinite vanishing points
         return;
     }
-    NR::Point p = vp.get_pos();
+    Geom::Point p = vp.get_pos();
 
-    for (GList *i = this->draggers; i != NULL; i = i->next) {
-        VPDragger *dragger = (VPDragger *) i->data;
-        if (NR::L2 (dragger->point - p) < MERGE_DIST) {
+    for (std::vector<VPDragger *>::const_iterator i = this->draggers.begin(); i != this->draggers.end(); ++i) {
+        VPDragger *dragger = *i;
+        if (Geom::L2 (dragger->point - p) < MERGE_DIST) {
             // distance is small, merge this draggable into dragger, no need to create new dragger
             dragger->addVP (vp);
             return;
@@ -734,16 +762,16 @@ VPDrag::addDragger (VanishingPoint &vp)
 
     VPDragger *new_dragger = new VPDragger(this, p, vp);
     // fixme: draggers should be added AFTER the last one: this way tabbing through them will be from begin to end.
-    this->draggers = g_list_append (this->draggers, new_dragger);
+    this->draggers.push_back(new_dragger);
 }
 
 void
 VPDrag::swap_perspectives_of_VPs(Persp3D *persp2, Persp3D *persp1)
 {
     // iterate over all VP in all draggers and replace persp2 with persp1
-    for (GList *i = this->draggers; i != NULL; i = i->next) {
-        for (std::list<VanishingPoint>::iterator j = ((VPDragger *) (i->data))->vps.begin();
-             j != ((VPDragger *) (i->data))->vps.end(); ++j) {
+    for (std::vector<VPDragger *>::const_iterator i = this->draggers.begin(); i != this->draggers.end(); ++i) {
+        for (std::list<VanishingPoint>::iterator j = (*i)->vps.begin();
+             j != (*i)->vps.end(); ++j) {
             if ((*j).get_perspective() == persp2) {
                 (*j).set_perspective(persp1);
             }
@@ -751,18 +779,11 @@ VPDrag::swap_perspectives_of_VPs(Persp3D *persp2, Persp3D *persp1)
     }
 }
 
-/**
-Create a line from p1 to p2 and add it to the lines list
- */
-void
-VPDrag::addLine (NR::Point p1, NR::Point p2, guint32 rgba)
+void VPDrag::addLine(Geom::Point const &p1, Geom::Point const &p2, Inkscape::CtrlLineType type)
 {
-    SPCanvasItem *line = sp_canvas_item_new(sp_desktop_controls(inkscape_active_desktop()), SP_TYPE_CTRLLINE, NULL);
-    sp_ctrlline_set_coords(SP_CTRLLINE(line), p1, p2);
-    if (rgba != VP_LINE_COLOR_FILL) // fill is the default, so don't set color for it to speed up redraw
-        sp_ctrlline_set_rgba32 (SP_CTRLLINE(line), rgba);
-    sp_canvas_item_show (line);
-    this->lines = g_slist_append (this->lines, line);
+    SPCtrlLine *line = ControlManager::getManager().createControlLine(SP_ACTIVE_DESKTOP->getControls(), p1, p2, type);
+    sp_canvas_item_show(line);
+    this->lines.push_back(line);
 }
 
 } // namespace Box3D
@@ -776,4 +797,4 @@ VPDrag::addLine (NR::Point p1, NR::Point p2, guint32 rgba)
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
